% Demonstration of M4Fun.m usage
% demoMGF estimates sample moments of real and artificial data in the wavelet domain. 
%
% jan.svihlik@vscht.cz
%
% If the presented m-functions and scripts will be used, cite:
%
% [1]  Svihlik, J., Fliegel, K., Kukal, J., Jerhotova, E., Pata, P., Vitek, S., and Koten, P., Estimation
%      of Non-Gaussian noise parameters in the wavelet domain using moment generating
%      function, Journal of Electronic Imaging 21(1) (2012).
%      DOI: 10.1117/1.JEI.21.3.039802


clear all
close all

k = menu('LOADING OF DATA','REAL','ARTIFICIAL');
switch(k)
    case(1)
data = double(fitsread('DARK_20_60_-082.fit'));    % real dark frame
data = data(1:504,1:760);
    case(2)
data = 10*rand(256,256);                           % uniformly distributed pseudo-random numbers
end
data = data - mean2(data);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  UNDECIMATED WAVELET TRANSFORM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

LenF = 2;                          % length of filter
h = daubcqf(LenF,'min');
L = 3;                             % set decomposition level
[LL3,yh,L] = mrdwt(data,h,L); 
 N = size(data,2);
   LH = yh(:,1:N); 
   HL = yh(:,N+1:2*N); 
   HH = yh(:,2*N+1:3*N);
   LH2 = yh(:,3*N+1:4*N); 
   HL2 = yh(:,4*N+1:5*N); 
   HH2 = yh(:,5*N+1:6*N);
   LH3 = yh(:,6*N+1:7*N); 
   HL3 = yh(:,7*N+1:8*N); 
   HH3 = yh(:,8*N+1:9*N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SAMPLE MOMENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

bandLL = LL3;                     % selected wavelet subbands
bandHH = HH3;

M2D  = moment(data(:),2)          % second moment will not be changed due to Parseval's theorem 
M4D  = moment(data(:),4)

disp('CENTRAL SAMPLE MOMENTS')

M2out(1) = M2D;
M4out(1) = M4D;

for i = 2:2*L+1
M4out(i) = M4Fun(M2D,M4out(i-1),h,num2str(LenF));
end

M2est = M2out(end)                 
M2LL = moment(bandLL(:),2)         % sample moments computed at given decomposition level using the definition
                                   % formula (DIRECT METHOD) Mk = (1/I)*suma(Ni)^k, , where I denotes a number of samples 
M4est = M4out(end)                 % moment evaluated using the proposed equations (PROPOSED METHOD)
M4LL = moment(bandLL(:),4) 





